@@ -40,9 +40,10 @@ extern "C" {
  * currently recognises the following object types :-
  *
  *  * Controls (mixer, enumerated and byte) including TLV data.
- *  * PCMs (FE and BE configurations and capabilities)
+ *  * PCMs (Front End DAI & DAI link)
  *  * DAPM widgets
  *  * DAPM graph elements.
+ *  * Physical DAI & DAI links
  *  * Private data for each object type.
  *  * Manifest (containing count of each object type)
  *
@@ -213,6 +214,20 @@ extern "C" {
  * The keyword tuples is to define vendor specific tuples. Please refer to
  * section Vendor Tokens and Vendor tuples.
  *
+ * It's easy to use a vendor tuples object to define a C structure instance.
+ * And a data section can include multiple vendor tuples objects:
+ *
+ * <pre>
+ * SectionData."data element name" {
+ *	index "1"	#Index number
+ *	tuples [
+ *		"id of the 1st vendor tuples section"
+ *		"id of the 2nd vendor tuples section"
+ *		...
+ *	]
+ * };
+ * </pre>
+ *
  * <h5>How to define an element with private data</h5>
  * An element can refer to a single data section or multiple data
  * sections.
@@ -355,9 +370,10 @@ extern "C" {
  * </pre>
  *
  * The section name is used to define the mixer name. The index number can be
- * used to identify topology objects groups. This allows driver operations on
- * objects with index number N and can be used to add/remove pipelines of
- * objects whilst other objects are unaffected.
+ * used to identify topology objects groups(index "0" is common, fit for all
+ * user cases).This allows driver operations on objects with index number N and
+ * can be used to add/remove pipelines of objects whilst other objects are
+ * unaffected.
  *
  * <h5>Byte Controls</h5>
  * A byte control is defined as a new section that can include channel mapping,
@@ -483,6 +499,7 @@ extern "C" {
  *	index "1"			# Index number
  *
  *	type "aif_in"			# Widget type - detailed above
+ *	stream_name "name"		# Stream name
  *
  *	no_pm "true"			# No PM control bit.
  *	reg "20"			# PM bit register offset
@@ -510,8 +527,8 @@ extern "C" {
  * refer to section Control Private Data.
  *
  * <h4>PCM Capabilities</h4>
- * Topology can also define the capabilities of FE and BE PCMs. Capabilities
- * can be defined with the following section :-
+ * Topology can also define the PCM capabilities of front end or physical DAIs.
+ * Capabilities can be defined with the following section :-
  *
  * <pre>
  * SectionPCMCapabilities."name" {
@@ -524,8 +541,8 @@ extern "C" {
  * }
  * </pre>
  * The supported formats use the same naming convention as the driver macros.
- * The PCM capabilities name can be referred to and included by BE, PCM and
- * Codec <-> codec topology sections.
+ * The PCM capabilities name can be referred to and included by PCM and
+ * physical DAI sections.
  *
  * <h4>PCM Configurations</h4>
  * PCM runtime configurations can be defined for playback and capture stream
@@ -551,31 +568,14 @@ extern "C" {
  * </pre>
  *
  * The supported formats use the same naming convention as the driver macros.
- * The PCM configuration name can be referred to and included by BE, PCM and
- * Codec <-> codec topology sections.
+ * The PCM configuration name can be referred to and included by PCM and
+ * physical link sections.
  *
- * <h4>PCM Configurations</h4>
- * PCM, BE and Codec to Codec link sections define the supported capabilities
- * and configurations for supported playback and capture streams. The
- * definitions and content for PCMs, BE and Codec links are the same with the
- * exception of the section type :-
- *
- * <pre>
- * SectionPCM."name" {
- *	....
- * }
- * SectionBE."name" {
- *	....
- * }
- * SectionCC."name" {
- *	....
- * }
- * </pre>
- *
- * The section types above should be used for PCMs, Back Ends and Codec to Codec
- * links respectively.<br>
- *
- * The data for each section is defined as follows :-
+ * <h4>PCM (Front-end DAI & DAI link) </h4>
+ * PCM sections define the supported capabilities and configurations for
+ * supported playback and capture streams, names and flags for front end
+ * DAI & DAI links. Topology kernel driver will use a PCM object to create
+ * a pair of FE DAI & DAI links.
  *
  * <pre>
  * SectionPCM."name" {
@@ -606,6 +606,89 @@ extern "C" {
  *			"config3"
  *		]
  *	}
+ *
+ *	# Optional boolean flags
+ *	symmetric_rates			"true"
+ *	symmetric_channels		"true"
+ *	symmetric_sample_bits		"false"
+ *
+ *	data "name"			# optional private data
+ * }
+ * </pre>
+ *
+ * <h4>Physical DAI Link Configurations</h4>
+ * The runtime configurations of a physical DAI link can be defined by
+ * SectionLink. <br> Backend DAI links belong to physical links, and can
+ * be configured by either SectionLink or SectionBE, with same syntax.
+ * But SectionBE is deprecated atm since the internal processing is
+ * actually same.
+ *
+ * <pre>
+ * SectionLink."name" {
+ *
+ *	index "1"			# Index number
+ *
+ *	id "0"				# used for binding to the link
+ *
+ *	stream_name "name"		# used for binding to the link
+ *
+ *	hw_configs [	# runtime supported HW configurations, optional
+ *		"config1"
+ *		"config2"
+ *		...
+ *	]
+ *
+ *	default_hw_conf_id "1"		#default HW config ID for init
+ *
+ *	# Optional boolean flags
+ *	symmetric_rates			"true"
+ *	symmetric_channels		"false"
+ *	symmetric_sample_bits		"true"
+ *
+ *	data "name"			# optional private data
+ * }
+ * </pre>
+ *
+ * A physical link can refer to multiple runtime supported hardware
+ * configurations, which is defined by SectionHWConfig.
+ *
+ * <pre>
+ * SectionHWConfig."name" {
+ *
+ *	id "1"				# used for binding to the config
+ *	format "I2S"			# physical audio format.
+ *	bclk   "master"			# Platform is master of bit clock
+ *	fsync  "slave"			# Platform is slave of fsync
+ * }
+ * </pre>
+ *
+ * <h4>Physical DAI</h4>
+ * A physical DAI (e.g. backend DAI for DPCM) is defined as a new section
+ * that can include a unique ID, playback and capture stream capabilities,
+ * optional flags, and private data. <br>
+ * Its PCM stream capablities are same as those for PCM objects,
+ * please refer to section 'PCM Capabilities'.
+ *
+ * <pre>
+ * SectionDAI."name" {
+ *
+ *	index "1"			# Index number
+ *
+ *	id "0"				# used for binding to the Backend DAI
+ *
+ *	pcm."playback" {
+ *		capabilities "capabilities1"	# capabilities for playback
+ *	}
+ *
+ *	pcm."capture" {
+ *		capabilities "capabilities2"	# capabilities for capture
+ *	}
+ *
+ *	symmetric_rates "true"			# optional flags
+ *	symmetric_channels "true"
+ *	symmetric_sample_bits "false"
+ *
+ *	data "name"			# optional private data
  * }
  * </pre>
  *
@@ -623,6 +706,36 @@ extern "C" {
  *	data "name"			# optional private data
  * }
  * </pre>
+ *
+ * <h4>Include other files</h4>
+ * Users may include other files in a text conf file via alsaconf syntax
+ * <path/to/configuration-file>. This allows users to define common info
+ * in separate files (e.g. vendor tokens, tuples) and share them for
+ * different platforms, thus save the total size of config files. <br>
+ * Users can also specifiy additional configuraiton directories relative
+ * to "/usr/share/alsa/" to search the included files,  via alsaconf syntax
+ * <searchfdir:/relative-path/to/usr/share/alsa>. <br><br>
+ *
+ * For example, file A and file B are two text conf files for platform X,
+ * they will be installed to /usr/share/alsa/topology/platformx. If we
+ * need file A to include file B, in file A we can add: <br>
+ *
+ * <searchdir:topology/platformx> <br>
+ * <name-of-file-B> <br><br>
+ *
+ * ALSA conf will search and open an included file in the following order
+ * of priority:
+ *  1. directly open the file by its name;
+ *  2. search for the file name in "/usr/share/alsa";
+ *  3. search for the file name in user specified subdirectories under
+ *     "/usr/share/alsa".
+ *
+ * The order of the included files need not to be same as their
+ * dependencies, since the topology library will load them all before
+ * parsing their dependencies. <br>
+ *
+ * The configuration directories defined by a file will only be used to search
+ * the files included by this file.
  */
 
 /** Maximum number of channels supported in one control */
@@ -649,8 +762,14 @@ enum snd_tplg_type {
 	SND_TPLG_TYPE_MANIFEST,		/*!< Topology manifest */
 	SND_TPLG_TYPE_TOKEN,		/*!< Vendor tokens */
 	SND_TPLG_TYPE_TUPLE,		/*!< Vendor tuples */
+	SND_TPLG_TYPE_LINK,		/*!< Physical DAI link */
+	SND_TPLG_TYPE_HW_CONFIG,	/*!< Link HW config */
+	SND_TPLG_TYPE_DAI,		/*!< Physical DAI */
 };
 
+/** Fit for all user cases */
+#define SND_TPLG_INDEX_ALL  0
+
 /**
  * \brief Create a new topology parser instance.
  * \return New topology parser instance
@@ -848,6 +967,7 @@ struct snd_tplg_stream_caps_template {
 	unsigned int period_size_max;	/*!< max period size bytes */
 	unsigned int buffer_size_min;	/*!< min buffer size bytes */
 	unsigned int buffer_size_max;	/*!< max buffer size bytes */
+	unsigned int sig_bits;		/*!< number of bits of content */
 };
 
 /** \struct snd_tplg_pcm_template
@@ -862,18 +982,74 @@ struct snd_tplg_pcm_template {
 	unsigned int capture;	/*!< supports capture mode */
 	unsigned int compress;	/*!< 1 = compressed; 0 = PCM */
 	struct snd_tplg_stream_caps_template *caps[2]; /*!< playback & capture for DAI */
+	unsigned int flag_mask; /*!< bitmask of flags to configure */
+	unsigned int flags;     /*!< flag value SND_SOC_TPLG_LNK_FLGBIT_* */
+	struct snd_soc_tplg_private *priv;	/*!< private data */
 	int num_streams;	/*!< number of supported configs */
 	struct snd_tplg_stream_template stream[0]; /*!< supported configs */
 };
 
+ /** \struct snd_tplg_hw_config_template
+ * \brief Template type to describe a physical link runtime supported
+ * hardware config, i.e. hardware audio formats.
+ */
+struct snd_tplg_hw_config_template {
+	int id;                         /* unique ID - - used to match */
+	unsigned int fmt;               /* SND_SOC_DAI_FORMAT_ format value */
+	unsigned char clock_gated;      /* 1 if clock can be gated to save power */
+	unsigned char  invert_bclk;     /* 1 for inverted BCLK, 0 for normal */
+	unsigned char  invert_fsync;    /* 1 for inverted frame clock, 0 for normal */
+	unsigned char  bclk_master;     /* 1 for master of BCLK, 0 for slave */
+	unsigned char  fsync_master;    /* 1 for master of FSYNC, 0 for slave */
+	unsigned char  mclk_direction;  /* 0 for input, 1 for output */
+	unsigned short reserved;        /* for 32bit alignment */
+	unsigned int mclk_rate;	        /* MCLK or SYSCLK freqency in Hz */
+	unsigned int bclk_rate;	        /* BCLK freqency in Hz */
+	unsigned int fsync_rate;        /* frame clock in Hz */
+	unsigned int tdm_slots;         /* number of TDM slots in use */
+	unsigned int tdm_slot_width;    /* width in bits for each slot */
+	unsigned int tx_slots;          /* bit mask for active Tx slots */
+	unsigned int rx_slots;          /* bit mask for active Rx slots */
+	unsigned int tx_channels;       /* number of Tx channels */
+	unsigned int *tx_chanmap;       /* array of slot number */
+	unsigned int rx_channels;       /* number of Rx channels */
+	unsigned int *rx_chanmap;       /* array of slot number */
+};
+
+/** \struct snd_tplg_dai_template
+ * \brief Template type for physical DAI.
+ * It can be used to configure backend DAIs for DPCM.
+ */
+struct snd_tplg_dai_template {
+	const char *dai_name;	/*!< DAI name */
+	unsigned int dai_id;	/*!< unique ID - used to match */
+	unsigned int playback;	/*!< supports playback mode */
+	unsigned int capture;	/*!< supports capture mode */
+	struct snd_tplg_stream_caps_template *caps[2]; /*!< playback & capture for DAI */
+	unsigned int flag_mask; /*!< bitmask of flags to configure */
+	unsigned int flags;	/*!< SND_SOC_TPLG_DAI_FLGBIT_* */
+	struct snd_soc_tplg_private *priv;	/*!< private data */
+
+};
+
 /** \struct snd_tplg_link_template
- * \brief Template type for BE and CC DAI Links.
+ * \brief Template type for physical DAI Links.
  */
 struct snd_tplg_link_template {
-	const char *name;	/*!< link name */
-	int id;	/*!< unique ID - used to match with existing BE and CC links */
+	const char *name;	/*!< link name, used to match */
+	int id;	/*!< unique ID - used to match with existing physical links */
+	const char *stream_name;        /*!< link stream name, used to match */
+
 	int num_streams;	/*!< number of configs */
-	struct snd_tplg_stream_template stream[0]; /*!< supported configs */
+	struct snd_tplg_stream_template *stream;       /*!< supported configs */
+
+	struct snd_tplg_hw_config_template *hw_config; /*!< supported HW configs */
+	int num_hw_configs;		/* number of hw configs */
+	int default_hw_config_id;       /* default hw config ID for init */
+
+	unsigned int flag_mask;         /* bitmask of flags to configure */
+	unsigned int flags;             /* SND_SOC_TPLG_LNK_FLGBIT_* flag value */
+	struct snd_soc_tplg_private *priv; /*!< private data */
 };
 
 /** \struct snd_tplg_obj_template
@@ -891,7 +1067,8 @@ typedef struct snd_tplg_obj_template {
 		struct snd_tplg_enum_template *enum_ctl;	/*!< Enum control */
 		struct snd_tplg_graph_template *graph;		/*!< Graph elements */
 		struct snd_tplg_pcm_template *pcm;		/*!< PCM elements */
-		struct snd_tplg_link_template *link;		/*!< BE and CC Links */
+		struct snd_tplg_link_template *link;		/*!< physical DAI Links */
+		struct snd_tplg_dai_template *dai;		/*!< Physical DAI */
 	};
 } snd_tplg_obj_template_t;
 
