@@ -41,7 +41,7 @@
 #include "use-case.h"
 
 #define MAX_FILE		256
-#define ALSA_USE_CASE_DIR	ALSA_CONFIG_DIR "/ucm"
+#define MAX_CARD_LONG_NAME	80
 
 #define SEQUENCE_ELEMENT_TYPE_CDEV	1
 #define SEQUENCE_ELEMENT_TYPE_CSET	2
@@ -190,6 +190,8 @@ struct use_case_verb {
  */
 struct snd_use_case_mgr {
 	char *card_name;
+	char card_long_name[MAX_CARD_LONG_NAME];
+	char conf_file_name[MAX_CARD_LONG_NAME];
 	char *comment;
 
 	/* use case verb, devices and modifier configs parsed from files */
